/* Copyright (C) 2006 by John Welch
 * Copyright (C) 2004 by Ted Phelps */
#ifndef MCMC_H
#define MCMC_H 1

/* The log likelihood computation function type.  You provide a
 * function of this type when calling mcmc.  It should return the log
 * of the likelihood of the data being observed for these values of
 * the parameters.
 *
 * index
 *    The index of the most recently modified parameter.  This can be
 *    useful for some kinds of likelihood function optimizations.  The
 *    very first time the likelihood function is called, this argument
 *    will be set to -1.
 *
 * last_used
 *    positive if the last likelihood value returned was kept, zero if
 *    it was discarded, negative the first time the function is called.
 *
 * param_count
 *    The number of parameters in the PARAMS array.
 *
 * params
 *    The current parameters.  This will be an array of doubles.  What
 *    they actually mean is up to the calling function and the
 *    likelihood function.
 *
 * rock
 *    The data supplied to MCMC as the likelihood_rock.
 */
typedef double (*mcmc_log_likelihood_func_t)(int index,
                                             int last_used,
                                             int param_count,
                                             double *params,
                                             void *rock);

/* The MCMC output function type.  You provide a function of this type
 * when calling MCMC, and MCMC will call it every 
 * PARAM_COUNT rounds so that you may record the walk.
 *
 * likelihood
 *    The likelihood estimate of the current set of parameters.
 *
 * param_count
 *    The number of parameters in the PARAMS array.
 *
 * params
 *    The current values of the parameters that the MCMC tweaks.  This
 *    is the same array that was passed to mcmc.  Modify at your peril.
 *
 * step_sizes
 *    The amount that the MCMC algorithm is varying the corresponding
 *    value in PARAMS.  Each time a parameter's value is changed, the
 *    step size is increased.  Conversely, each time a guess is
 *    discarded, the step size is decreased.
 *
 * rock
 *    The data supplied to MCMC as output_rock.
 */

/*
 * Monte Carlo Markov Chain computation

 * Estimates the values of a set of parameters, ensuring they are
 * always within the specified bounds.  The algorithm runs some number
 * of rounds.  In each round, one parameter is selected, its value is
 * tweaked and the resulting likelihood is computed.  If the new
 * likelihood is not substantially worse than the old (as determined
 * by the temperature), then the new value is kept and the search
 * domain for that parameter is widened.  Otherwise it is discarded
 * and the search domain is narrowed.
 *
 * t0
 *    The initial temperature for the simulated annealing
 *
 * dt
 *    Cooling rate: temperature = temperature * dt every "tscale" rounds
 *
 * tscale
 *    See above
 *
 * round_count
 *    The number of iterations to perform.
 *
 * param_count
 *    The number of parameters to estimate.  This is the number of
 *    items in each of the MINS, MAXS and PARAMS arrays.
 *
 * mins
 *    The lower bound for each of the PARAMS.  The MCMC function
 *    ensures that MINS[i] <= PARAMS[i] for each i.  There should be
 *    PARAM_COUNT items in the array.  Use -DBL_MAX to indicate that
 *    there is no lower bound.
 *
 * maxs
 *    The upper bound for each of the PARAMS.  The MCMC function
 *    ensures that PARAMS[i] <= MAXS[i] for each i.  There should be
 *    PARAM_COUNT items in the array.  Use DBL_MAX to indicate that
 *    there is no upper bound.
 *
 * indices
 *    If MCMC should adjust the parameters in a nondeterministic
 *    order, then indices should be an array of param_count integers.
 *    It should be NULL otherwise.
 *  
 * params
 *    The parameters to tweak.  When MCMC is called, the array should
 *    contain initial values for each of the parameters, upon
 *    completion it will contain the final estimates for them.  Better
 *    guesses for initial values will cause MCMC to converge more
 *    quickly.  There should be PARAM_COUNT values in the PARAMS
 *    array.
 *
 * likelihood_func
 *    A pointer to a function that computes the log of the likelihood
 *    value for a given set of PARAMS.  User data may be passed to
 *    this function through the LIKELIHOOD_ROCK argument.  This
 *    argument may not be NULL.
 *
 * likelihood_rock
 *    An opaque pointer to user data used in the computation of the
 *    log likelihood.  The MCMC function never looks at the value of
 *    this pointer; it merely passes it on to the LIKELIHOOD_FUNC.
 *
 */
int mcmc(double t0, double dt, int tscale,
          long round_count,
          int param_count,
          double *params,
          double *mins,
          double *maxs,
          double *step_sizes,
          int *indices,
          mcmc_log_likelihood_func_t likelihood_func,
          void *likelihood_rock);

#endif /* MCMC_H */